public inbox for gcc-cvs@sourceware.org
help / color / mirror / Atom feed
* [gcc r11-9049] Fix PR c++/64697 at -O1 or above
@ 2021-10-01  8:59 Eric Botcazou
  0 siblings, 0 replies; only message in thread
From: Eric Botcazou @ 2021-10-01  8:59 UTC (permalink / raw)
  To: gcc-cvs

https://gcc.gnu.org/g:618b7cb3b3fb4d2d92434d31ea8b6746ffef2572

commit r11-9049-g618b7cb3b3fb4d2d92434d31ea8b6746ffef2572
Author: Eric Botcazou <ebotcazou@adacore.com>
Date:   Fri Oct 1 10:49:34 2021 +0200

    Fix PR c++/64697 at -O1 or above
    
    The BFD fix eliminates the link failure and working code is generated at
    -O0, but _not_ when optimization is enabled because the optimizer changes:
    
            movq    .refptr._ZTH1s(%rip), %rax
            testq   %rax, %rax
            je      .L2
            call    _ZTH1s
    
    into:
    
            leaq    _ZTH1s(%rip), %rax
            testq   %rax, %rax
            je      .L2
            call    _ZTH1s
    
    and the leaq now also gets the relocation overflow.  So the fix is to
    teach legitimate_pic_address_disp_p to reject the transformation when
    the symbol is an external weak function, which yields:
    
            cmpq    $0, .refptr._ZTH1s(%rip)
            je      .L2
            call    _ZTH1s
    
    and the cmpq keeps a relocation that does not overflow.
    
    gcc/
            PR c++/64697
            * config/i386/i386.c (legitimate_pic_address_disp_p): For PE-COFF do
            not return true for external weak function symbols in medium model.

Diff:
---
 gcc/config/i386/i386.c | 21 ++++++++-------------
 1 file changed, 8 insertions(+), 13 deletions(-)

diff --git a/gcc/config/i386/i386.c b/gcc/config/i386/i386.c
index f991e07f232..fa80daba523 100644
--- a/gcc/config/i386/i386.c
+++ b/gcc/config/i386/i386.c
@@ -10567,24 +10567,19 @@ legitimate_pic_address_disp_p (rtx disp)
 	      if (is_imported_p (op0))
 		return true;
 
-	      if (SYMBOL_REF_FAR_ADDR_P (op0)
-		  || !SYMBOL_REF_LOCAL_P (op0))
+	      if (SYMBOL_REF_FAR_ADDR_P (op0) || !SYMBOL_REF_LOCAL_P (op0))
 		break;
 
-	      /* Function-symbols need to be resolved only for
-	         large-model.
-	         For the small-model we don't need to resolve anything
-	         here.  */
+	      /* Non-external-weak function symbols need to be resolved only
+		 for the large model.  Non-external symbols don't need to be
+		 resolved for large and medium models.  For the small model,
+		 we don't need to resolve anything here.  */
 	      if ((ix86_cmodel != CM_LARGE_PIC
-	           && SYMBOL_REF_FUNCTION_P (op0))
+		   && SYMBOL_REF_FUNCTION_P (op0)
+		   && !(SYMBOL_REF_EXTERNAL_P (op0) && SYMBOL_REF_WEAK (op0)))
+		  || !SYMBOL_REF_EXTERNAL_P (op0)
 		  || ix86_cmodel == CM_SMALL_PIC)
 		return true;
-	      /* Non-external symbols don't need to be resolved for
-	         large, and medium-model.  */
-	      if ((ix86_cmodel == CM_LARGE_PIC
-		   || ix86_cmodel == CM_MEDIUM_PIC)
-		  && !SYMBOL_REF_EXTERNAL_P (op0))
-		return true;
 	    }
 	  else if (!SYMBOL_REF_FAR_ADDR_P (op0)
 		   && (SYMBOL_REF_LOCAL_P (op0)


^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2021-10-01  8:59 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2021-10-01  8:59 [gcc r11-9049] Fix PR c++/64697 at -O1 or above Eric Botcazou

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).